--	LibBuilder
--	A tool for building multiple-file libraries


--	TODO: Implement better tracking for when libraries aren't closed properly

local MAJOR, MINOR = "LibBuilder-1.0", 1


local LibBuilder = LibStub:NewLibrary(MAJOR, MINOR)
if not LibBuilder then return end


--	used fields are listed here for documentation purposes
LibBuilder.workspace = nil
LibBuilder.major = nil
LibBuilder.minor = nil


---	Registers a new library and creates a workspace for it.
--	@param	major		The major version of the library being registered.
--	@param	minor		The minor version of the library being registered.
--	@return	workspace	The workspace for the new library.
function LibBuilder:NewLib(major, minor)
	assert(self.workspace == nil)
	assert(self.major == nil)
	assert(self.minor == nil)
	
	self.major = major
	self.minor = minor
	
	self.workspace = {}
	
	return self.workspace
end


---	Retrieves the version info and workspace for the current active library.
--	@return major		Major version of the library.
--	@return minor		Minor version of the library.
--	@return workspace	The workspace for the library.
function LibBuilder:GetLib()
	return self.major, self.minor, self.workspace
end


--	Closes the active library. This is used for tracking, to make sure 
--	that two libraries don't get clobbered together.
function LibBuilder:CloseLib()
	self.major, self.minor, self.workspace = nil, nil, nil
end
