--	Destination
--	


local major, minor, addonTable = LibStub("LibBuilder-1.0"):GetLib()
if not addonTable.canLoad then return end


local Destination, destination = addonTable:NewClass("Destination")


Destination.nameToFrame = {}		--	map of tab names to frame tables


destination.type = nil
destination.value = nil


---	Creates a new Destination object.
--	@return instance	The new object.
function Destination:New(value)
	local instance = {}
	setmetatable(instance, self.instanceMetatable)
	
	if value then
		instance:SetValue(value)
	end
	
	return instance
end


--	Internal; refreshes the mapping of tab names to frame objects
--	For now we're just going to call this each time a Print is requested
function Destination:RefreshTabMap()
	for i=1,NUM_CHAT_WINDOWS do
		local name, _, _, _, _, _, isShown, _, isDocked = GetChatWindowInfo(i)
		
		--	this is an old note, i think it means theres something interesting there
		--	NOTE: FCF_OpenNewWindow
		
		--	Skip windows that aren't visible since they 
		--	aren't condidered "available" anyway.
		--	Just remember that we have to watch when those attributes change.
		if (isShown or isDocked) then
			self.nameToFrame[name] = _G["ChatFrame"..i]
		end
	end
end


function Destination:GetFrame(tab)
	--	FIXME: hook something so we don't have to do this every time
	self:RefreshTabMap()
	
	return self.nameToFrame[tab]
end


--	Valid types:
--	nil			Generic, will guess type based on value.
--	tab			Accepts the name of a tab and prints to the associated frame.
--	frame		Accepts a frame object or index and prints to that frame.
--	callback	Accepts a function closure and passes the message to it.
function destination:SetType(newType)
	--	ALPHA: check type
	
	self.value = nil
	self.type = newType
end


function destination:GetType()
	if self.type ~= nil then
		return self.type
	end
	
	local type = type(self.value)
	
	if type == "string" then
		return "tab"
	elseif type == "function" then
		return "callback"
	elseif type == "table" or type == "number" then
		return "frame"
	end
end


function destination:SetValue(newValue)
	--	ALPHA: make sure it's appropriate for the current type
	
	self.value = newValue
end


function destination:IsValid()
	error("NYI")
end


function destination:IsAvailable()
	--print("destination:IsAvailable", self.type, self.value)
	
	local vType = type(self.value)
	
	if vType == nil then
		return false
	end
	
	local dType = self:GetType()
	
	if dType == "tab" then
		local frame = Destination:GetFrame(self.value)
		
		if frame then
			return true
		else
			return false
		end
	elseif dType == "callback" then
		if type(self.value) == "function" then
			return true
		else
			return false
		end
	elseif dType == "frame" then
		if self.value then
			return true
		else
			return false
		end
	end
end


function destination:SetExclusive(isExclusive)
	self.isExclusive = isExclusive
end


function destination:IsExclusive()
	return self.isExclusive or true
end


function destination:SetRaw(isRaw)
	self.isRaw = isRaw
end


function destination:IsRaw()
	return self.isRaw or false
end


function destination:Print(message)
	local type = self:GetType()
	
	if type == "tab" then
		Destination:GetFrame(self.value):AddMessage(message)
	elseif type == "callback" then
		if self:IsRaw() then
			self.value(unpack(message))
		else
			self.value(message)
		end
	elseif type == "frame" then
		local frame = self.value
		
		if _G.type(frame) == "number" then
			frame = _G["ChatFrame"..frame]
		end
		
		frame:AddMessage(msg)
	end
end
