--	LibPrint
--	


local major, minor, addonTable = LibStub("LibBuilder-1.0"):GetLib()
if not addonTable.canLoad then return end


local LibPrint = addonTable.interface


LibPrint.messageCache = {}
LibPrint.tabsAvailable = false


function LibPrint:NewStream(...)
	local stream = addonTable.classes.Stream:New()
	
	--	FIXME: nil arg in ... will cause loop to stop
	for i, arg in ipairs({...}) do
		local destination = self:NewDestination(arg)
		stream:AddDestination(destination)
	end
	
	return stream
end


function LibPrint:NewDestination(value)
	return addonTable.classes.Destination:New(value)
end


function LibPrint:AreTabsAvailable()
	if GetChatWindowInfo(1) == "" then
		return false
	else
		return true
	end
end


function LibPrint:UPDATE_CHAT_WINDOWS()
	if table.maxn(LibPrint.messageCache) > 0 then
		--print("Dumping cached messages")
		for i, data in ipairs(LibPrint.messageCache) do
			data.stream:Print(data.message)
		end
		--print("Finished dumping cached messages")
	end
end


function LibPrint:Hook()
	local frame = CreateFrame("Frame")
	frame:SetScript("OnEvent", function(...) self:UPDATE_CHAT_WINDOWS(...) end)
	frame:RegisterEvent("UPDATE_CHAT_WINDOWS")
end


function LibPrint:QueueMessage(message, stream)
	if table.maxn(self.messageCache) == 0 then
		print("LibPrint: caching messages")
	end
	table.insert(self.messageCache, { message = message, stream = stream })
end
	


LibPrint:NewStream("LibPrint", "mcm", "debug", print):Print("loaded", major, "Revision", minor)


--	EXPERIMENTAL! SUBJECT TO CHANGE
function LibPrint:PrintPath()
	local stack = debugstack()
	local line = string.match(stack, "\n(.-)\n")
	--	FIXME: if path gets truncated...
	local shortPath = string.match(line, "Interface\\AddOns\\(.-):")	--	FIXME: other locales?
	print("LibPrint:", "running", shortPath or line)
end

LibPrint:PrintPath()


